import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:intl_phone_number_input/intl_phone_number_input.dart';
import 'package:pin_input_text_field/pin_input_text_field.dart';
import 'package:slide_countdown_clock/slide_countdown_clock.dart';
import 'package:firebase_phone_verify_ui/bloc/app_bloc.dart';



enum OTPStsatus {
  loading,
  error,
  completed,
  otp,
}

enum ValidateStsatus {
  sendingSMS,
  phoneInvalide,
  couldNotSentCode,
  smsSentCompleted,
  codeVerifiedCompleted,
  verifying,
  invalideOTP,
  error,
}

class OTPScreen extends StatefulWidget {
  final PhoneNumber phoneNumber;

  OTPScreen(this.phoneNumber);

  _OTPScreenState createState() => _OTPScreenState();
}

class _OTPScreenState extends State<OTPScreen> {
  String _appVersion = '1.0';
  String uuid;
  AppBloc bloc;
  String phoneIsoCode = "+855";
  String _verificationId;
  OTPStsatus _status = OTPStsatus.loading;
  ValidateStsatus _valideStatus = ValidateStsatus.sendingSMS;
  bool _allowResent = false;

  final _editController = TextEditingController();
  final FirebaseAuth _auth = FirebaseAuth.instance;

  @override
  void initState() {
    super.initState();
    bloc = BlocProvider.of<AppBloc>(context);

    Future.delayed(Duration(milliseconds: 500), () {
      _sendSMSCode();
      //_loginWithPhone("+85593401361");
    });
  }

  @override
  void setState(VoidCallback fn) {
    if (mounted) {
      super.setState(fn);
    }
  }

  _verifyCode() {
    setState(() {
      _status = OTPStsatus.loading;
      _valideStatus = ValidateStsatus.verifying;
    });

    var credential = PhoneAuthProvider.getCredential(
        verificationId: _verificationId, smsCode: _editController.text);
    _auth.signInWithCredential(credential).then((result) {
      setState(() {
        _valideStatus = ValidateStsatus.codeVerifiedCompleted;
      });
      //_loginWithPhone(widget.phoneNumber.phoneNumber);
      print("_______signInWithCredential___________success_________$result");
    }).catchError((onError) {
      setState(() {
        _valideStatus = ValidateStsatus.invalideOTP;
      });
      print("_______signInWithCredential___________error_________$onError");
    });
  }

  _codeSent(String verificationId, [int n]) {
    setState(() {
      _verificationId = verificationId;
      _valideStatus = ValidateStsatus.smsSentCompleted;
    });

    Future.delayed(Duration(milliseconds: 500), () {
      setState(() {
        _status = OTPStsatus.otp;
      });
    });

    print("____code_sent_completd");
  }

  _verificationCompleted(AuthCredential authCredential) {
    print("_veryfyCodeCompleted__$authCredential");
    setState(() {
      _valideStatus = ValidateStsatus.smsSentCompleted;
    });
    Future.delayed(Duration(milliseconds: 200), (){
      setState(() {
        _status = OTPStsatus.otp;
      });
    });

    Future.delayed(Duration(milliseconds: 200), (){
      setState(() {
        _valideStatus = ValidateStsatus.smsSentCompleted;
      });
    });

    //_loginWithPhone(widget.phoneNumber.phoneNumber);
  }

  _codeAutoRetrievalTimeout(String s) {
    print("__codeAutoRetrievalTimeout____$s");
  }

  _verificationFailed(AuthException f) {
    setState(() {
      _status = OTPStsatus.loading;
      _valideStatus = ValidateStsatus.error;
    });

    print("_verificationFailed_______${f.code}________${f.message}");
  }

  _sendSMSCode() async {
    String phone = widget.phoneNumber.phoneNumber;
    _auth.verifyPhoneNumber(
      phoneNumber: phone,
      timeout: Duration(minutes: 2),
      codeSent: _codeSent,
      verificationCompleted: _verificationCompleted,
      codeAutoRetrievalTimeout: _codeAutoRetrievalTimeout,
      verificationFailed: _verificationFailed,
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        resizeToAvoidBottomPadding: false,
        appBar: AppBar(
          title: Text("OTP"),
          centerTitle: true,
        ),
        backgroundColor: Colors.white,
        body: Builder(
          builder: (context) {
            switch (_status) {
              case OTPStsatus.loading:
                return _buildLoadingScreen();
                break;

              case OTPStsatus.completed:
                return _buildOTPScreen();
                break;

              case OTPStsatus.otp:
                return _buildOTPScreen();
                break;
              case OTPStsatus.error:
                return _buildOTPScreen();
                break;

              default:
                return Container();
            }
          },
        ));
  }

  Stack _buildLoadingScreen() {

    return Stack(
      children: <Widget>[
        Padding(
          padding: const EdgeInsets.only(top: 32, left: 16, right: 16),
          child: Container(
              width: double.infinity,
              child: Builder(
                builder: (context) {
                  switch (_valideStatus) {
                    case ValidateStsatus.sendingSMS:
                      return Column(
                        mainAxisSize: MainAxisSize.min,
                        children: <Widget>[
                          Text("Sending SMS with your code.",
                              style: TextStyle(fontSize: 17)),
                          SizedBox(height: 16),
                          Container(
                            width: 70,
                            height: 70,
                            child: CircularProgressIndicator(),
                          ),
                        ],
                      );
                      break;

                    case ValidateStsatus.verifying:
                      return Column(
                        mainAxisSize: MainAxisSize.min,
                        children: <Widget>[
                          Text("Verifying!", style: TextStyle(fontSize: 17)),
                          SizedBox(height: 16),
                          Container(
                            width: 70,
                            height: 70,
                            child: CircularProgressIndicator(),
                          ),
                        ],
                      );
                      break;

                    case ValidateStsatus.invalideOTP:
                      return Column(
                        mainAxisSize: MainAxisSize.min,
                        children: <Widget>[
                          Text("Invalide code!",
                              style: TextStyle(fontSize: 17)),
                          SizedBox(height: 24),
                          Container(
                            width: 100,
                            height: 100,
                            child: Icon(Icons.error_outline, size: 100),
                          ),
                          SizedBox(height: 24),
                          Container(
                            width: double.infinity,
                            color: Colors.red,
                            child: FlatButton(
                              child: Text("Try again",
                                  style: TextStyle(color: Colors.white)),
                              onPressed: () {
                                setState(() {
                                  _status = OTPStsatus.otp;
                                });
                              },
                            ),
                          ),
                        ],
                      );
                      break;

                    case ValidateStsatus.smsSentCompleted:
                      return Column(
                        mainAxisSize: MainAxisSize.min,
                        children: <Widget>[
                          Text("Code was sent.",
                              style: TextStyle(fontSize: 17)),
                          SizedBox(height: 16),
                          Container(
                            width: 100,
                            height: 100,
                            child: Icon(Icons.check_circle_outline, size: 100),
                          ),
                        ],
                      );
                      break;

                    case ValidateStsatus.codeVerifiedCompleted:
                      return Column(
                        mainAxisSize: MainAxisSize.min,
                        children: <Widget>[
                          Text("Phone verified.",
                              style: TextStyle(fontSize: 17)),
                          SizedBox(height: 16),
                          Container(
                            width: 100,
                            height: 100,
                            child: Icon(Icons.check_circle_outline, size: 100),
                          ),
                        ],
                      );
                      break;

                    case ValidateStsatus.phoneInvalide:
                      return Column(
                        mainAxisSize: MainAxisSize.min,
                        children: <Widget>[
                          Text("Please enter a valid phone number.",
                              style: TextStyle(fontSize: 17)),
                          SizedBox(height: 24),
                          Container(
                            width: 100,
                            height: 100,
                            child: Icon(Icons.error_outline, size: 100),
                          ),
                          SizedBox(height: 24),
                          Container(
                            width: double.infinity,
                            color: Colors.red,
                            child: FlatButton(
                              child: Text("Try again",
                                  style: TextStyle(color: Colors.white)),
                              onPressed: () {
                                Navigator.pop(context);
                              },
                            ),
                          ),
                        ],
                      );
                      break;
                    case ValidateStsatus.error:
                      return Column(
                        mainAxisSize: MainAxisSize.min,
                        children: <Widget>[
                          Text("Something went wrong.",
                              style: TextStyle(fontSize: 17)),
                          SizedBox(height: 24),
                          Container(
                            width: 100,
                            height: 100,
                            child: Icon(Icons.error_outline, size: 100),
                          ),
                          SizedBox(height: 24),
                          FlatButton(
                            child: Text("Try again"),
                            onPressed: () {
                              Navigator.pop(context);
                            },
                          ),
                        ],
                      );
                      break;

                    default:
                      return Container();
                      break;
                  }
                },
              )),
        ),
      ],
    );
  }

  Stack _buildOTPScreen() {
    return Stack(
      children: <Widget>[
        Padding(
          padding: const EdgeInsets.only(top: 16, left: 32, right: 32),
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              Text("Verification code", style: TextStyle(fontSize: 27)),
              SizedBox(height: 24),
              Text("Please enter the OTP sent on your registered phone number.",
                  textAlign: TextAlign.center),
              SizedBox(height: 16),
              PinInputTextField(
                controller: _editController,
                pinLength: 6,
                decoration: BoxLooseDecoration(
                    hintText: "------",
                    enteredColor: Colors.red,
                    strokeColor: Colors.red),
                focusNode: FocusNode(),
                onChanged: (t) {
                  print(t);
                  setState(() {});
                },
              ),
              SizedBox(height: 32),
              !_allowResent
                  ? SlideCountdownClock(
                      duration: Duration(minutes: 2),
                      slideDirection: SlideDirection.Down,
                      separator: ":",
                      shouldShowDays: false,
                      textStyle: TextStyle(
                        color: Colors.black38,
                        fontSize: 27,
                        fontWeight: FontWeight.normal,
                      ),
                      onDone: () {
                        setState(() {
                          _allowResent = true;
                        });
                      },
                    )
                  : 
                  Row(
                      mainAxisAlignment: MainAxisAlignment.center,
                      mainAxisSize: MainAxisSize.min,
                      children: <Widget>[
                        Text("Didn't you received any code?", style: TextStyle(fontSize: 12)),
                        FlatButton(
                          child: Text("Resent OTP", style: TextStyle(color: Colors.red)),
                          onPressed: () {
                          _sendSMSCode();
                          setState(() {
                            _status = OTPStsatus.loading;
                            _valideStatus = ValidateStsatus.sendingSMS;
                            _allowResent = false;
                          });
                          },
                        )
                        
                      ],
                    ),
              SizedBox(height: 24),
              Container(
                width: double.infinity,
                height: 44,
                child: RaisedButton(
                  child: Text("Submit", style: TextStyle(color: Colors.white)),
                  onPressed:
                      _editController.text.length == 6 ? _verifyCode : null,
                ),
              )
            ],
          ),
        ),
      ],
    );
  }
}
